
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "base/ObjId.h"
#include "client/PrivCache.h"


PrivCache::PrivCache(int aCapacity) {
	if (aCapacity > 0)
		capacity(aCapacity);
}

void PrivCache::capacity(int aCap) {
	Assert(aCap >= 0);
	Assert(!theHash.count());

	theHistory.resize(aCap);

	// would be nice to find the closest prime
	aCap = 2*aCap | 1;
	Assert(aCap <= 0xFFFF); // we use short int as an index
	theHash.stretch(2*aCap | 1);
	theHash.count(theHash.capacity());
}

Time PrivCache::stamp(const ObjId &oid) const {
	const PrivCacheEntry &e = theHash[hash(oid)];
	if (e && e.theName == oid.name() &&	e.theWorld == oid.world().hash())
		return Time::Sec(e.theStampSecs);
	return Time();
}

void PrivCache::putOid(const ObjId &oid, Time stamp) {
	Assert(oid.name() > 0);

	if (theHistory.full())
		purgeOne();

	const int idx = hash(oid);
	PrivCacheEntry &e = theHash[idx];
	if (e) // collision
		return;

	e.theName = oid.name();
	e.theWorld = oid.world().hash();
	e.theStampSecs = stamp.sec();

	theHistory.enqueue((IdxItem)idx);
}

int PrivCache::hash(const ObjId &oid) const {
	return oid.hash() % theHash.capacity();
}

void PrivCache::purgeOne() {
	const int idx = theHistory.dequeue();
	theHash[idx].theName = -1;
}
