
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_EMBEDDED_CONTENT_MODEL_H
#define PG_EMBEDDED_CONTENT_MODEL_H

#include "xstd/Array.h"
#include "base/ObjId.h"

class WrBuf;
class RndGen;
class RndDistr;
class BodyIter;
class ContentCfg;
class ContentSym;


// simulates embedded content 
// (simulates various *ML tags that allow to "embed" content, to be precise)

class EmbedContMdl {
	public:
		EmbedContMdl();

		void configure(const ContentSym *cfg);

		// called by Server through ContentCfg
		double compContPerCall(const ContentCfg *cc) const;
		void noteNewContProb(ContentCfg *cc, double newProb);

		// called by ContainerBodyIter
		int embedGoal(RndGen &rng);
		ObjId embedOid(RndGen &rng, const ObjId &oid, int oidCount);

	protected:
		void configureEmbedCfgs(const ContentSym *cfg);
		bool find(int cfgId, int &idx) const;

	protected:
		Array<const ContentCfg*> theCfgs; // embedded objs configs
		Array<double> theProbs;       // probability of selecting a cfg
		Array<double> theNewContProb; // prob. of producing new content when producing an embedded oid
		RndDistr *theCfgSel;          // selector for cfgs
		RndDistr *theEmbedCount;      // objs per container
		
		double theEmbedCountMean;     // cached mean of theEmbedCount
};

#endif
