
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/h/strstream.h"

#include "pgl/PglNetAddrSym.h"
#include "pgl/PglNetAddrRange.h"
#include "pgl/PglNetAddrRangeSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/IpsToNames.h"


ArraySym *IpsToNames(const ArraySym &ips, const String &dname) {

	Array<PglNetAddrRange*> ranges;
	const int ipCount = ips.count();
	char buf[1024];
	for (int i = 0; i < ipCount; ++i) {
		const NetAddrSym &ipSym = (const NetAddrSym&)ips.item(i)->cast("addr");
		const NetAddr &ip = ipSym.val();
		ostrstream os(buf, sizeof(buf));
		os << "w[" << 1000 + ip.octet(4)
			<< "].h[" << 1000 + ip.octet(3)
			<< "]o[" << 1000 + ip.octet(2) 
			<< "]s[" << 1000 + ip.octet(1)
			<< "]." << dname;
		if (ip.port() >= 0)
			os << ':' << ip.port();
		os << ends;
		PglNetAddrRange r;
		Assert(r.parse(buf));

		// if can merge r with last range, add new range if failed
		if (!ranges.count() || !ranges.last()->canMergeAddrs(r))
			ranges.append(new PglNetAddrRange);

		Assert(ranges.count());
		ranges.last()->merge(r);
	}

	// XXX: unify with same code in VerFourAs!
	// merge ranges if possible
	bool merged = true;
	do {
		merged = false;
		for (int left = 0, right = 1; !merged && right < ranges.count(); ++right) {
			PglNetAddrRange *&r = ranges[right];
			if (!r) // deleted by earlier merges
				continue;
			if (ranges[left] && ranges[left]->canMergeAddrs(*r)) {
				ranges[left]->merge(*r);
				delete r;
				r = 0;
				merged = true;
			} else {
				left = right;
			}
		}
	} while (merged);

	// convert an array of ranges into ArraySym
	ArraySym *names = new ArraySym("addr");
	{for (int i = 0; i < ranges.count(); ++i) {
		if (!ranges[i])
			continue;
		NetAddrRangeSym rs;
		rs.range(ranges[i]);
		names->add(rs);
	}}

	return names;
}
