
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/h/strstream.h"

#include "xstd/String.h"
#include "xstd/NetAddr.h"
#include "xstd/gadgets.h"
#include "pgl/PglNetAddrSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglNetAddrRange.h"
#include "pgl/pglStrIs.h"


PglNetAddrRange::PglNetAddrRange(): theSubnet(-1) {
}

void PglNetAddrRange::reset() {
	PglStrRange::reset();
	theAddr = 0;
	theIfName = 0;
	theSubnet = -1;
}

void PglNetAddrRange::netmask(struct in_addr &netm) const {
	int subnet = theSubnet; // explicitly specified subnet, if any

	if (subnet < 0)
		subnet = rangedBlockCnt() > 1 ? 16 : 24;

	netm = NetAddr::NetMask(subnet);
}

ArraySym *PglNetAddrRange::toSyms(const TokenLoc &loc) const {
	ArraySym *arr = new ArraySym("addr");

	startIter();
	do {
		NetAddrSym nas;
		currentAddrSym(nas);
		if (loc)
			nas.loc(loc);
		arr->add(nas); // XXX: replace with add(this)

	} while (nextIter());

	return arr;
}

void PglNetAddrRange::toAddrs(Array<NetAddr*> &addrs) const {
	startIter();
	do {
		String str;
		currentIter(str);

		const NetAddr addr = pglIsNetAddr(str);
		Assert(addr);

		addrs.append(new NetAddr(addr));
	} while (nextIter());
}

void PglNetAddrRange::toAddrs(AddrIter iter) const {
	startIter();
	do {
		String str;
		currentIter(str);

		NetAddr addr = pglIsNetAddr(str);
		Assert(addr);

		iter(addr);
	} while (nextIter());
}

void PglNetAddrRange::addrAt(int idx, NetAddrSym &nas) const {
	startIter();
	skipIter(idx);
	currentAddrSym(nas);
}

void PglNetAddrRange::currentAddrSym(NetAddrSym &nas) const {
	String str;
	currentIter(str);

	NetAddr addr = pglIsNetAddr(str);
	Assert(addr);

	nas.val(addr);
	if (theIfName)
		nas.setIfname(theIfName);
	if (theSubnet >= 0)
		nas.setSubnet(theSubnet);
}

bool PglNetAddrRange::parse(const String &val) {
	if (!pglSplitAddr(val, theIfName, theAddr, theSubnet))
		return false;

	return PglStrRange::parse(theAddr);
}

bool PglNetAddrRange::canMergeAddrs(const PglNetAddrRange &r) const {
	if (!theImage) // brand new object
		return true;

	if ((theIfName || r.theIfName) && theIfName != r.theIfName)
		return false;

	if (theSubnet != r.theSubnet)
		return false;

	return PglStrRange::canMergeStrs(r);
}

bool PglNetAddrRange::optRangeBeg(char ch) const {
	return ch == '.' || ch == ':';
}

ostream &PglNetAddrRange::print(ostream &os) const {
	if (theIfName)
		os << theIfName << "::";
	
	PglStrRange::print(os);

	if (theSubnet >= 0)
		os << '/' << theSubnet;

	return os;
}
