
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglArraySym.h"
#include "pgl/BenchSym.h"
#include "pgl/BenchSideSym.h"
#include "pgl/SrvLb4AsSym.h"


SrvLb4AsSym::SrvLb4AsSym(): VerFourAsSym("SrvLb4As", new PglRec) {
	kind("SrvLb-4");
}

SrvLb4AsSym::SrvLb4AsSym(const String &aType, PglRec *aRec): VerFourAsSym(aType, aRec) {
}

bool SrvLb4AsSym::isA(const String &type) const {
	return VerFourAsSym::isA(type) || type == "SrvLb4As";
}

SynSym *SrvLb4AsSym::dupe(const String &type) const {
	if (isA(type))
		return new SrvLb4AsSym(this->type(), theRec->clone());
	return VerFourAsSym::dupe(type);
}

String SrvLb4AsSym::robots(ArraySym *&addrs) const {
	return addresses(theBench->clientSide(), addrs);
}

String SrvLb4AsSym::servers(ArraySym *&addrs) const {
	if (const ArraySym *space = theBench->serverSide()->addrSpace()) {
		cerr << space->loc() << "warning: SrvLb4As addressing scheme"
			<< " uses server_side.hosts only; your setting of"
			<< " server_side.addr_space will be ignored" << endl;
	}

	if (ArraySym *hosts = theBench->serverSide()->hostsSym()) {
		if (hosts->count()) {
			addrs = &((ArraySym&)hosts->clone()->cast("addr[]"));
			return 0;
		}
	}

	return "no server_side.hosts specified";
}

String SrvLb4AsSym::proxies(ArraySym *&addrs) const {
	return VerFourAsSym::proxies(addrs);
}
